package ssd;


import java.io.FileWriter;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.transform.Result;
import javax.xml.transform.Source;
import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.dom.DOMSource;
import javax.xml.transform.stream.StreamResult;

import org.w3c.dom.DOMImplementation;
import org.w3c.dom.Document;
import org.w3c.dom.NamedNodeMap;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;
import org.w3c.dom.ls.DOMImplementationLS;
import org.w3c.dom.ls.LSOutput;
import org.w3c.dom.ls.LSSerializer;



public class DOMDemo {
	
	private final static String TYPEMAPPING[] = {"UNDEFINED_TYPE","ELEMENT","ATTRIBUTE","TEXT","CDATA","ENTITY REF","ENTITY","PI","COMMENT","DOCUMENT","DTD","DOCUMENT FRAG","NOTATION"};


	public static void main(String[] args) throws Exception {
		DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
		
    		factory.setValidating(true);
  		factory.setNamespaceAware(true);
  		factory.setIgnoringComments(false);
    		factory.setIgnoringElementContentWhitespace(true);
		
		DocumentBuilder db = factory.newDocumentBuilder();
		db.setErrorHandler(new DOMErrorHandler());
		Document doc = db.parse(args[0]);
		
 		traverse(doc,0);

//		BAUM MANIPULIEREN
//   		duplicate(doc);
// 		traverse(doc,0);
  		
//		BAUM SCHREIBEN
//   		outputTransformer(doc, "filenameDOMTransformer.xml");
//  		outputLSSerializer(doc, "filenameLSSerializer.xml");
	}
	
	public static void traverse(Node n, int level) {
		printNode(n, level);
		
		if(n.hasAttributes()) {
			NamedNodeMap atts = n.getAttributes();
			for(int i = 0; i < atts.getLength(); ++i) {
				Node a = atts.item(i);
				printNode(a, level);
			}
		}
		
		NodeList children = n.getChildNodes();
		for(int i = 0; i < children.getLength(); ++i) {
			traverse(children.item(i), level + 1);
		}
	}

	public static void printNode(Node n, int level) {
		print(		n.getNodeName()
				+ "\t"
				+ n.getLocalName()
				+ "\t"
				+ n.getNamespaceURI()
				+ "\t"
				+ TYPEMAPPING[n.getNodeType()]
				+ ":\t\""
				+ n.getNodeValue()
				+ "\"", level);
	}

	private static void print(String s, int level) {
		String str = s
			.replaceAll("\n", "<EOL>")
			.replaceAll("\r", ""); // ignore \r only (used by old Apple OS)
		for(int i=0; i < level; ++i) System.out.print(" ");
		System.out.println(str);
	}
	
	public static void duplicate(Node n) {
		NodeList children = n.getChildNodes();
		for(int i = 0; i < children.getLength(); ++i) {
			Node c = children.item(i);
			
			if(c.getNodeType() == Node.ELEMENT_NODE && c.getNodeName().equals("TITEL")) {
				Node childClone = c.cloneNode(true); // deep
				n.insertBefore(childClone, c);
				++i; // neu eingefuegtes Element ueberspringen
			}
			
			if(c.getNodeType() == Node.ELEMENT_NODE && c.getNodeName().equals("AUTOR")) {
				Node childClone = c.cloneNode(false); // not deep
				n.insertBefore(childClone, c);
				++i; // neu eingefuegtes Element ueberspringen
			}
			
			duplicate(c);
		}
	}
	
	public static void outputTransformer(Document doc, String filename) throws Exception {
		TransformerFactory tf = TransformerFactory.newInstance();
		Transformer t = tf.newTransformer();
		
		Source ds = new DOMSource(doc);
		Result sink = new StreamResult(filename);
		
		t.transform(ds, sink);
	}
	
	public static void outputLSSerializer(Document doc, String filename) throws Exception {
		DOMImplementation di = doc.getImplementation();
		if (di.hasFeature("Core","3.0")){
			DOMImplementationLS diLS = (DOMImplementationLS) di;
			LSSerializer lss = diLS.createLSSerializer();
			
			String xmlDoc = lss.writeToString(doc);
			
			FileWriter sink = new FileWriter(filename);
			sink.write(xmlDoc);  // achtung! UTF-16!
			sink.flush();
			sink.close(); 
		} else {
			// does not support LSSerializer
		}
	}
	
}
