package ssd;

import org.xml.sax.Attributes;
import org.xml.sax.Locator;
import org.xml.sax.SAXException;
import org.xml.sax.SAXParseException;
import org.xml.sax.helpers.DefaultHandler;

public class SSDHandler extends DefaultHandler {

	private Locator locator;
	private int level = 0;
	private StringBuffer sb = new StringBuffer();
	private boolean charUpdated = false;
	
	/********************[ ContentHandler ]**********************/
	
	public void startDocument() throws SAXException {
		level = 0;
		sb = new StringBuffer();
		charUpdated = false;
		print("startDocument");
	}

	public void startElement(String uri, String localName, String name,
			Attributes attributes) throws SAXException {
		printText();
		++level;
		print("startElement: "
				+localName
				+" "
				+uri
				+" "
				+name);
		printAttributes(attributes);
	}

	public void characters(char[] ch, int start, int length) throws SAXException {
		charUpdated = true;
		for(int i=0; i<length; ++i)
			sb.append(ch[start+i]);
	}
	
	public void ignorableWhitespace(char[] ch, int start, int length) throws SAXException { 
		StringBuffer s = new StringBuffer();
		for(int i=0; i<length; ++i)
			sb.append(ch[start+i]);
		System.out.println(" --> IGNORED: \"" + s.toString() + "\"");
	}

	public void endElement(String uri, String localName, String name)
			throws SAXException {
		printText();
		print("endElement: "
				+localName
				+" "
				+uri
				+" "
				+name);
		--level;
	}
	
	public void endDocument() throws SAXException {
		print("endDocument");
		--level;
	}
	
	/********************[ Locator ]**********************/
	
	public void setDocumentLocator(Locator loc) {
		locator = loc;
	}
	
	/********************[ Error handling ]**********************/
	
	public void warning(SAXParseException e) throws SAXException {
		printError("WARNING",e);
	}
	
	public void error(SAXParseException e) throws SAXException {
		printError("ERROR",e);
	}
	
	public void fatalError(SAXParseException e) throws SAXException {
		printError("FATAL",e);
	}
	
	/********************[ Helpers ]**********************/
	
	/**
	 * Print the string with indentation appropriate to the current level
	 */
	private void print(String s) {
		System.out.printf("%3d,%3d ", locator.getLineNumber(), locator.getColumnNumber());
		for(int i=0; i < level; ++i) System.out.print("  ");
		System.out.println(s);
	}
	
	private void printAttributes(Attributes atts) {
		for(int i=0; i<atts.getLength(); ++i) {
			print("ATTRIBUTE: " 
					+ atts.getLocalName(i) 
					+ " "
					+ atts.getQName(i) 
					+ " "
					+ atts.getType(i) 
					+ " "
					+ atts.getURI(i) 
					+ " \"" 
					+ atts.getValue(i) 
					+ "\"");
		}
	}
	
	/**
	 * Print text content, but make EOL readable
	 */
	private void printText() {
		if(charUpdated) {
			if(level > 0) { // there is no text in front of the root element
				String str = (sb.toString())
					.replaceAll("\n", "<EOL>")
					.replaceAll("\r", ""); // ignore \r only (used by old Apple OS)
				print("Text: \"" + str + "\"");
			}
		}
		sb = new StringBuffer(); // empty the buffer
		charUpdated = false;
	}
	
	private void printError(String severity, SAXParseException e) {
		System.out.printf(" --> %s at %3d,%3d: %s\n"
				, severity
				, e.getLineNumber()
				, e.getColumnNumber()
				, e.getMessage());
	}
}
